import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

import math
import pickle

"""
liste_premiers = [2]

def premiers_premiers(n, liste_premiers):
    for i in range(3, n+1, 2):
        if est_premier(i, liste_premiers):
            liste_premiers.append(i)

def est_premier(n, liste_premiers):
    racine = int(math.sqrt(n))
    i = 0
    while liste_premiers[i] <= racine:
        if n % liste_premiers[i] == 0:
            return False
        i += 1
    return True
"""

# Dimension de la fenêtre initiale
XMIN, XMAX = -10, 10
YMIN, YMAX = -10, 10

coeff = 1.05  # À chaque étape on aggrandit la fenêtre de 5%
taille_point = 12  # Taille des points à l'étape 1

t_max = 250  # Nombre d'étapes

coeff_max = coeff**t_max  # Coefficient multiplicateur à la dernière étape

# taille du plus grand nombre qui pourra être affiché dans les coins
limite_max = math.sqrt((XMAX*coeff_max)**2+(YMAX*coeff_max)**2)

# taille maximale de la fenêtre
xmin_max, xmax_max = XMIN*coeff_max, XMAX*coeff_max
ymin_max, ymax_max = YMIN*coeff_max, YMAX*coeff_max

print(f"La limite est {limite_max}")
def lit_premiers(nom_fichier="liste_premiers.dat"):
    with open(nom_fichier, "rb") as fichier:
        return pickle.load(fichier)

# on ne garde que les nombres premiers nécessaires
liste_premiers = [p for p in lit_premiers("liste_premiers_500millions.dat") if p <= limite_max]
#premiers_premiers(int(limite_max)+1, liste_premiers)
print(f"On a {len(liste_premiers)} nombres premiers dont le plus grand est {liste_premiers[-1]}")

seuil_nombres_conjugues = 150

restes = []  # Les sous-spirales à afficher
modulo = 6  # Le nombre qui génère les sous-spirales

# Création de la figure
fig, ax = plt.subplots()
fig.set_figheight(7)
fig.set_figwidth(7)

# Les 2 axes
axe_horizontal, = ax.plot([xmin_max, xmax_max], [0, 0], color='grey');
axe_vertical, = ax.plot([0, 0], [ymin_max, ymax_max], color='grey');

# Pour tracer les sous-spirales
liste_points_restes = []
for i in range(len(restes)):
    points_restes, = ax.plot([], [])
    liste_points_restes.append(points_restes)

points_base, = ax.plot([], [], 'go', markersize=taille_point) # Les points des entiers non premiers
points, = ax.plot([], [], 'ro', markersize=taille_point) # Les points des nombres premiers

# On fixe la taille du repère dans la figure
ax.set_ylim(YMIN, YMAX)
ax.set_xlim(XMIN, XMAX)

ax.set_aspect('equal', adjustable='box') # Repère orthonormé

# Pour placer un point
def ajouter_point(n, xdata, ydata):
    x, y = n*math.cos(n), n*math.sin(n)
    xdata.append(x)
    ydata.append(y)

# On calcul la transparence des points "normaux"
# val, c'est l'étape où on est
# decal c'est la valeur centrale de la partie de transition
# c, c'est pour que la zone de transition soit plus ou moins courte
def transparence(val, decal, c): 
    return 1/(1+math.exp(c*(val-decal)))

limite = limite_max # On va calculer la position de tous les points, même non affichés
xdata, ydata = [], []
xdata2, ydata2 = [], []
xdata3, ydata3 = [0], [0]
# On trace les sous-spirales
for k in range(len(restes)):
    xdata3, ydata3 = [0], [0]
    for i in range(restes[k], int(limite)+1, modulo):
        ajouter_point(i, xdata3, ydata3)
    liste_points_restes[k].set_data(xdata3, ydata3)
# On place les points des entiers premiers ou pas
precedent = 0
for p in liste_premiers:
    for n in range(precedent, p):
        ajouter_point(n, xdata2, ydata2)
    ajouter_point(p, xdata, ydata)
    precedent = p + 1
points_base.set_data(xdata2, ydata2)
points.set_data(xdata, ydata)

# Mise à jour du zoom
def mise_a_jour(t):
    c = coeff**t
    xmin, xmax = XMIN*c, XMAX*c
    ymin, ymax = YMIN*c, YMAX*c
    ax.set_xlim(xmin, xmax)
    ax.set_ylim(ymin, ymax)
    # On calcule la taille des points
    points_base.set_markersize(max(taille_point/(math.sqrt(c)), 0.5))
    points.set_markersize(max(taille_point/(math.sqrt(c)), 1))
    # On calcule le niveau de transparence des axes
    axe_horizontal.set_alpha(max(0, 1-2*t/t_max))
    axe_vertical.set_alpha(max(0, 1-2*t/t_max))
    # On calcule le niveau de transparence des points normaux
    points_base.set_alpha(transparence(t, seuil_nombres_conjugues, 0.5)) # Les points disparaîssent autour de l'étape 150
    ax.figure.canvas.draw()

# Le slider pour régler l'étape
axcolor = 'lightgoldenrodyellow'
axfreq = plt.axes([0.2, 0.04, 0.6, 0.03], facecolor=axcolor)
sfreq = Slider(axfreq, 'etape', 0, t_max, valinit=0, valstep=1)
sfreq.on_changed(mise_a_jour)

# On fait la première mise à jour
mise_a_jour(0)

plt.show()